/*----------------------------------------------------------------------------
 *      Name:    serial.c
 *      Purpose: serial port handling for MCB2300
 *      Version: V1.03
 *      NOTE:    Tx is only buffered in the HW fifo (max 16 bytes)
 *----------------------------------------------------------------------------
 *      This file is part of the uVision/ARM development tools.
 *      This software may only be used under the terms of a valid, current,
 *      end user licence from KEIL for a compatible version of KEIL software
 *      development tools. Nothing else gives you the right to use it.
 *
 *      Copyright (c) 2005-2007 Keil Software.
 *---------------------------------------------------------------------------*/
 
#include <LPC23xx.H>                                     // LPC23xx definitions
#include "serial.h"

#define UART_CLK                   (24000000UL)          // UART Clock is 24.0 MHz

#define SER_IN_BUF_SIZE            64                    // Input buffer in bytes (power 2)
#define SER_OUT_BUF_SIZE           0                     // Output buffer in bytes (power 2)


#define SER_PORTCLOSED             0
#define SER_PORTOPENED             1

#define SER_MINPORT                0
#define SER_MAXPORT                1

#define SER_OK                     1
#define SER_GENERAL_ERR            0
#define SER_PORTNOTKNOWN_ERR       (-1)
#define SER_PORTNOTOPENED_ERR      (-1)
#define SER_DEVNOTKNOWN_ERR        (-1)

/* Buffer masks */
#define SER_IN_MASK                (SER_IN_BUF_SIZE-1ul)
#define SER_OUT_MASK               (SER_OUT_BUF_SIZE-1ul)

/* Buffer read / write macros */
#define OUT_BUF_RESET(serDev)      (serDev->ser_out.rd_idx = serDev->ser_out.wr_idx = 0)
#define OUT_BUF_WR(serDev, dataIn) (serDev->ser_out.data[SER_OUT_MASK & serDev->ser_out.wr_idx++] = (dataIn))
#define OUT_BUF_RD(serDev)         (serDev->ser_out.data[SER_OUT_MASK & serDev->ser_out.rd_idx++])   
#define OUT_BUF_EMPTY(serDev)      (serDev->ser_out.rd_idx == serDev->ser_out.wr_idx)
#define OUT_BUF_FULL(serDev)       (serDev->ser_out.rd_idx == serDev->ser_out.wr_idx+1)

#define IN_BUF_RESET(serDev)       (serDev->ser_in.rd_idx = serDev->ser_in.wr_idx = 0)
#define IN_BUF_WR(serDev, dataIn)  (serDev->ser_in.data[SER_IN_MASK & serDev->ser_in.wr_idx++] = (dataIn))
#define IN_BUF_RD(serDev)          (serDev->ser_in.data[SER_IN_MASK & serDev->ser_in.rd_idx++])   
#define IN_BUF_EMPTY(serDev)       (serDev->ser_in.rd_idx == serDev->ser_in.wr_idx)
#define IN_BUF_FULL(serDev)        (serDev->ser_in.rd_idx == serDev->ser_in.wr_idx+1)
#define IN_BUF_COUNT(serDev)       (SER_IN_MASK & (serDev->ser_in.wr_idx - serDev->ser_in.rd_idx))


// Serial output buffer
#if 0 // Not implemented
typedef struct __ser_out_t {
  unsigned char data[SER_OUT_BUF_SIZE];
  unsigned int wr_idx;
  unsigned int rd_idx;
} ser_out_t;
#endif


// Serial input buffer
typedef struct __ser_in_t {
  unsigned char data[SER_IN_BUF_SIZE];
  unsigned int wr_idx;
  unsigned int rd_idx;
} ser_in_t;


/*----------------------------------------------------------------------------
  structure to administrate the serial port
 *---------------------------------------------------------------------------*/
typedef __irq void (*ser_irq_fp)(void);                  // RV irq function pointer typedef
static __irq void ser_irq_0 (void);                      // RV irq function prototype
static __irq void ser_irq_2 (void);                      // RV irq function prototype

typedef struct {
                    int   serialDeviceNumber;            // = portNumber + 1
           unsigned int   status;
           unsigned long  baudRate;
           unsigned int   dataBits;
           unsigned int   stopBits;
           unsigned int   parity;
  volatile unsigned long *rbr;
  volatile unsigned long *thr;
  volatile unsigned long *dll;
  volatile unsigned long *dlm;
  volatile unsigned long *lcr;
  volatile unsigned long *lsr;
  volatile unsigned long *msr;
  volatile unsigned long *fdr;
  volatile unsigned long *fcr;
  volatile unsigned long *iir;
  volatile unsigned long *ier;
  volatile unsigned long *irqAddr;
  volatile unsigned long *irqCntl;  
  ser_irq_fp             ser_irq;                        // Serial IRQ function
  unsigned long          ser_irq_mask;                   // Serial IRQ enable / disable bit mask
  unsigned short         ser_lineState;                  // ((msr << 8) | (lsr))
  unsigned long          txFifoEmpty      :1;            // 0x000(0000000X) Indicates transmit fifo is empty and needs to be kickstarted
  unsigned long          unused_flags     :31;           // 0xXXX(XXXXXXX0) Unused 

#if 0 // Not implemented
  ser_out_t              ser_out;                        // Serial data buffers
#endif

  ser_in_t               ser_in;
} SER_INFO;

static SER_INFO ser_Dev[SER_MAXPORT + 1] = {
  { 1, 
    SER_PORTCLOSED, 9600, 8, 0, 0, 
    (volatile unsigned long *)(UART0_BASE_ADDR + 0x00),  // RBR
    (volatile unsigned long *)(UART0_BASE_ADDR + 0x00),  // THR
    (volatile unsigned long *)(UART0_BASE_ADDR + 0x00),  // DLL
    (volatile unsigned long *)(UART0_BASE_ADDR + 0x04),  // DLM
    (volatile unsigned long *)(UART0_BASE_ADDR + 0x0C),  // LCR
    (volatile unsigned long *)(UART0_BASE_ADDR + 0x14),  // LSR
    (volatile unsigned long *)(UART0_BASE_ADDR + 0x14),  // MSR dummy
    (volatile unsigned long *)(UART0_BASE_ADDR + 0x28),  // FDR
    (volatile unsigned long *)(UART0_BASE_ADDR + 0x08),  // FCR
    (volatile unsigned long *)(UART0_BASE_ADDR + 0x08),  // IIR
    (volatile unsigned long *)(UART0_BASE_ADDR + 0x04),  // IER
    (volatile unsigned long *)(VIC_BASE_ADDR + 0x118),   // IRQ Addr
    (volatile unsigned long *)(VIC_BASE_ADDR + 0x218),   // IRQ Cntl
    ser_irq_0,                                           // IRQ function
    0x00000040,                                          // IRQ mask bit
	0                                                    // ser_lineState
  },
  { 2, 
    SER_PORTCLOSED, 9600, 8, 0, 0, 
    (volatile unsigned long *)(UART2_BASE_ADDR + 0x00),  // RBR	 
    (volatile unsigned long *)(UART2_BASE_ADDR + 0x00),  // THR	
    (volatile unsigned long *)(UART2_BASE_ADDR + 0x00),  // DLL	 
    (volatile unsigned long *)(UART2_BASE_ADDR + 0x04),  // DLM	 
    (volatile unsigned long *)(UART2_BASE_ADDR + 0x0C),  // LCR	 
    (volatile unsigned long *)(UART2_BASE_ADDR + 0x14),  // LSR	  
    (volatile unsigned long *)(UART2_BASE_ADDR + 0x14),  // MSR	dummy
    (volatile unsigned long *)(UART2_BASE_ADDR + 0x28),  // FDR	
    (volatile unsigned long *)(UART2_BASE_ADDR + 0x08),  // FCR	
    (volatile unsigned long *)(UART2_BASE_ADDR + 0x08),  // IIR	
    (volatile unsigned long *)(UART2_BASE_ADDR + 0x04),  // IER	
    (volatile unsigned long *)(VIC_BASE_ADDR + 0x170),   // IRQ Addr
    (volatile unsigned long *)(VIC_BASE_ADDR + 0x270),   // IRQ Cntl
    ser_irq_2,                                           // IRQ function
    0x10000000,                                          // IRQ mask bit
	0                                                    // ser_lineState
  }
};

/*----------------------------------------------------------------------------
  Function Prototypes
 *---------------------------------------------------------------------------*/



/*----------------------------------------------------------------------------
  open the serial port
 *---------------------------------------------------------------------------*/
int ser_OpenPort (int port, int *serDevNo) {

  if ( (port < SER_MINPORT) ||                           // check the port number
       (port > SER_MAXPORT)   ) {
    return (SER_PORTNOTKNOWN_ERR);
  }

  if (port == 0) 
  {
	PINSEL0 &= 0xFFFFFF0F;									// Reset P0.2,P0.3 Pin Config
    PINSEL0 |= 0x00000010;									// Select P0.2 = TxD(UART0)
    PINSEL0 |= 0x00000040;									// Select P0.3 = RxD(UART0)
  }
  else 
  {    
	PINSEL0 &= 0xFF0FFFFF;									// Reset P0.10,P0.11 Pin Config
    PINSEL0 |= 0x00100000;									// Select P0.10 = TxD(UART2)
    PINSEL0 |= 0x00400000;									// Select P0.11 = RxD(UART2)
	PCONP   |= 0x01000000;									// UART2 Power-ON
  }

  ser_Dev[port].status = SER_PORTOPENED;
  *serDevNo = ser_Dev[port].serialDeviceNumber;

  return (SER_OK);
}

/*----------------------------------------------------------------------------
  close the serial port
 *---------------------------------------------------------------------------*/
int ser_ClosePort (int serDevNo) {
  SER_INFO *pSerDev;

  if ( (serDevNo < (SER_MINPORT+1)) ||                   // check the serial device number
       (serDevNo > (SER_MAXPORT+1))   ) {
    return (SER_DEVNOTKNOWN_ERR);
  }

  pSerDev = &ser_Dev[(serDevNo-1)];

  if ((serDevNo-1) == 0) 
  {    
	PINSEL0 &= 0xFFFFFF0F;									// Reset P0.2,P0.3 Pin Config    
  }
  else 
  {    
	PINSEL0 &= 0xFF0FFFFF;									// Reset P0.10,P0.11 Pin Config    
	PCONP   &= ~0x01000000;									// UART2 Power-OFF
  }

  /* Disable the interrupt in the VIC and UART controllers */
  *(pSerDev->ier) = 0x00;                                // Disable UART interrupts
  VICIntEnClr = pSerDev->ser_irq_mask;                   // Disable VIC interrupt

  pSerDev->status   = SER_PORTCLOSED;
  
  return (SER_OK);
}

/*----------------------------------------------------------------------------
  initialize the serial port
 *---------------------------------------------------------------------------*/
int ser_InitPort (int serDevNo, unsigned long baudrate, unsigned int  databits,
                                unsigned int  parity,   unsigned int  stopbits) {

  unsigned char lcr_p, lcr_s, lcr_d;
  unsigned long dll;
  SER_INFO *pSerDev;

  if ( (serDevNo < (SER_MINPORT+1)) ||                   // check the serial device number
       (serDevNo > (SER_MAXPORT+1))   ) {
    return (SER_DEVNOTKNOWN_ERR);
  }

  pSerDev = &ser_Dev[(serDevNo-1)];

  switch (databits) {
    case 5:                                              // 5 Data bits
      lcr_d = 0x00;
    break;
    case 6:                                              // 6 Data bits
      lcr_d = 0x01;
    break;
    case 7:                                              // 7 Data bits
      lcr_d = 0x02;
    break;
    case 8:                                              // 8 Data bits
    default:
      lcr_d = 0x03;
    break;
  }

  switch (stopbits) {
    case 1:                                              // 1,5 Stop bits
    case 2:                                              // 2   Stop bits
      lcr_s = 0x04;
    break;
    case 0:                                              // 1   Stop bit
    default:
      lcr_s = 0x00;
    break;
  }

  switch (parity) {
    case 1:                                              // Parity Odd
      lcr_p = 0x08;
    break;
    case 2:                                              // Parity Even
      lcr_p = 0x18;
    break;
    case 3:                                              // Parity Mark
      lcr_p = 0x28;
    break;
    case 4:                                              // Parity Space
      lcr_p = 0x38;
    break;
    case 0:                                              // Parity None
    default:
      lcr_p = 0x00;
    break;
  }

  pSerDev->baudRate = baudrate;
  pSerDev->dataBits = databits;
  pSerDev->stopBits = stopbits;
  pSerDev->parity   = parity;
  
  //OUT_BUF_RESET(pSerDev);
  IN_BUF_RESET(pSerDev);
  
  /* Note that the pclk is 24,0 MHz.  (48.0 MHz / 2)         */
  /* 24 MHz PCLK generates also rates for 115200, 57600 baud */
  dll = ((UART_CLK / baudrate) / 16UL);

  *(pSerDev->fdr) = 0;                                   // Fractional divider not used
  *(pSerDev->lcr) = 0x80 | lcr_d | lcr_p | lcr_s;        // Data bits, Parity,   Stop bit
  *(pSerDev->dll) = dll;                                 // Baud Rate @ 24 MHZ PCLK
  *(pSerDev->dlm) = (dll >> 8);                          // High divisor latch
  *(pSerDev->lcr) = 0x00 | lcr_d | lcr_p | lcr_s;        // DLAB = 0
  *(pSerDev->fcr) = 0x87;                                // Enable and reset FIFOs (irq=8 Chr in RxFIFO)
  *(pSerDev->ier) = 0x03;                                // Enable TX/RX interrupts
  pSerDev->txFifoEmpty = 1;                              // TX fifo is empty

  /* Set up and enable the UART interrupt in the VIC */
  *(pSerDev->irqAddr) = (unsigned long)pSerDev->ser_irq; // Set interrupt function
  *(pSerDev->irqCntl) = 0x0F;                            // Set interrupt priority
  VICIntEnable = pSerDev->ser_irq_mask;                  // Enable interrupt 

  return (SER_OK);
}

/*----------------------------------------------------------------------------
  read data from serial port
 *---------------------------------------------------------------------------*/
int ser_Read (int serDevNo, char *buffer, const int *length) {
  SER_INFO *pSerDev;
  int bytesToRead, bytesRead;
  
  if ( (serDevNo < (SER_MINPORT+1)) ||                   // check the serial device number
       (serDevNo > (SER_MAXPORT+1))   ) {
    return (SER_DEVNOTKNOWN_ERR);
  }
  pSerDev = &ser_Dev[(serDevNo-1)];

  /* Read *length bytes, block if *bytes are not avaialable	*/
  bytesToRead = *length;
  bytesToRead = (bytesToRead < (*length)) ? bytesToRead : (*length);
  bytesRead = bytesToRead;

  while (bytesToRead--) {
    while (IN_BUF_EMPTY(pSerDev));                       // Block until data is available if none
    *buffer++ = IN_BUF_RD(pSerDev);
  }
  return (bytesRead);  
}

/*----------------------------------------------------------------------------
  write data to the serial port
 *---------------------------------------------------------------------------*/
int ser_Write (int serDevNo, const char *buffer, int *length) {
  SER_INFO *pSerDev;
  int i, bytesToWrite, bytesWritten;

  if ( (serDevNo < (SER_MINPORT+1)) ||                   // check the serial device number
       (serDevNo > (SER_MAXPORT+1))   ) {
    return (SER_DEVNOTKNOWN_ERR);
  }
  pSerDev = &ser_Dev[(serDevNo-1)];

  // Write *length bytes
  bytesToWrite = *length;
  bytesWritten = bytesToWrite;

  /* Lets just use the 16 byte hardware FIFO to buffer outgoing data */
  while (bytesToWrite) {
    if (pSerDev->txFifoEmpty) {
      for (i=16; (bytesToWrite && i);i--) {
        pSerDev->txFifoEmpty = 0;
        *(pSerDev->thr) = *buffer++;                     // Write to the Tx FIFO
        bytesToWrite--;
      }      
    }
    // Else: Wait for the fifo to be empty
  }

  return (bytesWritten); 
}

/*----------------------------------------------------------------------------
  check if character(s) are available at the serial interface
 *---------------------------------------------------------------------------*/
int ser_AvailChar (int serDevNo, int *availChar) {
  SER_INFO *pSerDev;

  if ( (serDevNo < (SER_MINPORT+1)) ||                   // check the serial device number
       (serDevNo > (SER_MAXPORT+1))   ) {
    return (SER_DEVNOTKNOWN_ERR);
  }
  pSerDev = &ser_Dev[(serDevNo-1)];

  *availChar = IN_BUF_COUNT(pSerDev);

  return (SER_OK);
}

/*----------------------------------------------------------------------------
  read the line state of the serial port
 *---------------------------------------------------------------------------*/
int ser_LineState (int serDevNo, unsigned short *lineState) {
  SER_INFO *pSerDev;

  if ( (serDevNo < (SER_MINPORT+1)) ||                   // check the serial device number
       (serDevNo > (SER_MAXPORT+1))   ) {
    return (SER_DEVNOTKNOWN_ERR);
  }
  pSerDev = &ser_Dev[(serDevNo-1)];

  *lineState = pSerDev->ser_lineState;
  pSerDev->ser_lineState = 0;

  return (SER_OK); 
}

/*----------------------------------------------------------------------------
  serial port 0 interrupt
 *---------------------------------------------------------------------------*/
static __irq void ser_irq_0 (void) {
  SER_INFO *pSerDev = &ser_Dev[0];
  volatile unsigned long iir;

  iir = *(pSerDev->iir);
   
  switch (iir & 0x000F) {
    case 0x4:                                            // RDA pending
    case 0xC:                                            // CTI pending
      while (*(pSerDev->lsr) & 0x01) {                   // Rx FIFO is not empty
        IN_BUF_WR(pSerDev, *(pSerDev->rbr));             // Read Rx FIFO to buffer  
      }
      break;
    
    case 0x2:                                            // THRE pending
        pSerDev->txFifoEmpty = 1;
      break;
    
    default:
      ; // Ignore interrupt
  }

  pSerDev->ser_lineState = (*(pSerDev->lsr)) & 0xE01E;	// UART 0 has no MSR register
 
  VICVectAddr = 0;
}

/*----------------------------------------------------------------------------
  serial port 2 interrupt
 *---------------------------------------------------------------------------*/
static __irq void ser_irq_2 (void) { 
  SER_INFO *pSerDev = &ser_Dev[1];
  volatile unsigned long iir;
  
  iir = *(pSerDev->iir);
   
  switch (iir & 0x000F) {
    case 0x4:                                            // RDA pending
    case 0xC:                                            // CTI pending
      while (*(pSerDev->lsr) & 0x01) {                   // Rx FIFO is not empty
        IN_BUF_WR(pSerDev, *(pSerDev->rbr));             // Read Rx FIFO to buffer  
      }
      break;
    
    case 0x2:                                            // THRE pending
        pSerDev->txFifoEmpty = 1;
      break;
    
    default:
      ; // Ignore interrupt
  }
  pSerDev->ser_lineState = (*(pSerDev->lsr)) & 0xE01E;	// UART 2 has no MSR register
  
  VICVectAddr = 0;
}



/*----------------------------------------------------------------------------
  The following functions are used by the printf / scanf if stdout is
  redirected to the serial port.
 *---------------------------------------------------------------------------*/

#ifndef UART_PRINTF_BAUDRATE 
#define UART_PRINTF_BAUDRATE 115200
#endif


#ifndef UART_PRINTF_PORT 
#define UART_PRINTF_PORT 1
#endif

static int printfDevNo;

void init_serial (void)  {                               // Initialize Serial Interface

  ser_OpenPort (UART_PRINTF_PORT, &printfDevNo);
  ser_InitPort(printfDevNo,
               UART_PRINTF_BAUDRATE,
               8,   /* 8 data bits */ 
               0,   /* No partity */ 
               0);  /* 1 stop bit */
}

/* implementation of putchar (also used by printf function to output data)    */
#define CR     0x0D
int sendchar (int ch)  {                                 // Write character to Serial Port
  int length = 1;
  int cr = CR;

  if (ch == '\n')  {
    ser_Write (printfDevNo, (const char *)&cr, &length);
  }
  ser_Write (printfDevNo, (const char *)&ch, &length);

  return (ch);
}


int getkey (void)  {                                     // Read character from Serial Port
  int availChar;
  int length = 1;
  char data;

  do {
    ser_AvailChar(printfDevNo, &availChar);
  } while (!availChar);

  ser_Read (printfDevNo, &data, &length);

  return (data);
}



